@@ -1,5 +1,5 @@
-/* java.util.Calendar
-   Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+/* Calendar.java --
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -38,11 +38,12 @@ exception statement from your version. */
 
 package java.util;
 
-import java.lang.reflect.InvocationTargetException;
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.Serializable;
+import java.lang.reflect.Constructor;
+import java.lang.reflect.InvocationTargetException;
 
 /**
  * This class is an abstract base class for Calendars, which can be
@@ -50,7 +51,7 @@ import java.io.Serializable;
  * integer fields which represent <code>YEAR</code>,
  * <code>MONTH</code>, <code>DAY</code>, etc.  The <code>Date</code>
  * object represents a time in milliseconds since the Epoch. <br>
- * 
+ *
  * This class is locale sensitive.  To get the Object matching the
  * current locale you can use <code>getInstance</code>.  You can even provide
  * a locale or a timezone.  <code>getInstance</code> returns currently
@@ -77,13 +78,13 @@ import java.io.Serializable;
  * and for the first line all fields are set, that line is used to
  * compute the day. <br>
  *
- * 
+ *
 <pre>month + day_of_month
 month + week_of_month + day_of_week
 month + day_of_week_of_month + day_of_week
 day_of_year
 day_of_week + week_of_year</pre>
- * 
+ *
  * The hour_of_day-field takes precedence over the ampm and
  * hour_of_ampm fields. <br>
  *
@@ -91,7 +92,7 @@ day_of_week + week_of_year</pre>
  *
  * To convert a calendar to a human readable form and vice versa,  use
  * the <code>java.text.DateFormat</code> class. <br>
- * 
+ *
  * Other useful things you can do with an calendar, is
  * <code>roll</code>ing fields (that means increase/decrease a
  * specific field by one, propagating overflows), or
@@ -100,7 +101,7 @@ day_of_week + week_of_year</pre>
  * @see Date
  * @see GregorianCalendar
  * @see TimeZone
- * @see java.text.DateFormat 
+ * @see java.text.DateFormat
  */
 public abstract class Calendar implements Serializable, Cloneable
 {
@@ -108,43 +109,52 @@ public abstract class Calendar implements Serializable, Cloneable
    * Constant representing the era time field.
    */
   public static final int ERA = 0;
+
   /**
    * Constant representing the year time field.
    */
   public static final int YEAR = 1;
+
   /**
    * Constant representing the month time field.  This field
    * should contain one of the JANUARY,...,DECEMBER constants below.
    */
   public static final int MONTH = 2;
+
   /**
    * Constant representing the week of the year field.
    * @see #setFirstDayOfWeek(int)
    */
   public static final int WEEK_OF_YEAR = 3;
+
   /**
    * Constant representing the week of the month time field.
    * @see #setFirstDayOfWeek(int)
    */
   public static final int WEEK_OF_MONTH = 4;
+
   /**
    * Constant representing the day time field, synonym for DAY_OF_MONTH.
    */
   public static final int DATE = 5;
+
   /**
    * Constant representing the day time field.
    */
   public static final int DAY_OF_MONTH = 5;
+
   /**
    * Constant representing the day of year time field.  This is
    * 1 for the first day in month.
    */
   public static final int DAY_OF_YEAR = 6;
+
   /**
    * Constant representing the day of week time field.  This field
    * should contain one of the SUNDAY,...,SATURDAY constants below.
    */
   public static final int DAY_OF_WEEK = 7;
+
   /**
    * Constant representing the day-of-week-in-month field.  For
    * instance this field contains 2 for the second thursday in a
@@ -152,42 +162,51 @@ public abstract class Calendar implements Serializable, Cloneable
    * from the end of the month.
    */
   public static final int DAY_OF_WEEK_IN_MONTH = 8;
+
   /**
    * Constant representing the part of the day for 12-hour clock.  This
    * should be one of AM or PM.
    */
   public static final int AM_PM = 9;
+
   /**
    * Constant representing the hour time field for 12-hour clock.
    */
   public static final int HOUR = 10;
+
   /**
    * Constant representing the hour of day time field for 24-hour clock.
    */
   public static final int HOUR_OF_DAY = 11;
+
   /**
    * Constant representing the minute of hour time field.
    */
   public static final int MINUTE = 12;
+
   /**
    * Constant representing the second time field.
    */
   public static final int SECOND = 13;
+
   /**
    * Constant representing the millisecond time field.
    */
   public static final int MILLISECOND = 14;
+
   /**
    * Constant representing the time zone offset time field for the
    * time given in the other fields.  It is measured in
-   * milliseconds.  The default is the offset of the time zone.  
+   * milliseconds.  The default is the offset of the time zone.
    */
   public static final int ZONE_OFFSET = 15;
+
   /**
    * Constant representing the daylight saving time offset in
-   * milliseconds.  The default is the value given by the time zone.  
+   * milliseconds.  The default is the value given by the time zone.
    */
   public static final int DST_OFFSET = 16;
+
   /**
    * Number of time fields.
    */
@@ -197,26 +216,32 @@ public abstract class Calendar implements Serializable, Cloneable
    * Constant representing Sunday.
    */
   public static final int SUNDAY = 1;
+
   /**
    * Constant representing Monday.
    */
   public static final int MONDAY = 2;
+
   /**
    * Constant representing Tuesday.
    */
   public static final int TUESDAY = 3;
+
   /**
    * Constant representing Wednesday.
    */
   public static final int WEDNESDAY = 4;
+
   /**
    * Constant representing Thursday.
    */
   public static final int THURSDAY = 5;
+
   /**
    * Constant representing Friday.
    */
   public static final int FRIDAY = 6;
+
   /**
    * Constant representing Saturday.
    */
@@ -226,50 +251,62 @@ public abstract class Calendar implements Serializable, Cloneable
    * Constant representing January.
    */
   public static final int JANUARY = 0;
+
   /**
    * Constant representing February.
    */
   public static final int FEBRUARY = 1;
+
   /**
    * Constant representing March.
    */
   public static final int MARCH = 2;
+
   /**
    * Constant representing April.
    */
   public static final int APRIL = 3;
+
   /**
    * Constant representing May.
    */
   public static final int MAY = 4;
+
   /**
    * Constant representing June.
    */
   public static final int JUNE = 5;
+
   /**
    * Constant representing July.
    */
   public static final int JULY = 6;
+
   /**
    * Constant representing August.
    */
   public static final int AUGUST = 7;
+
   /**
    * Constant representing September.
    */
   public static final int SEPTEMBER = 8;
+
   /**
    * Constant representing October.
    */
   public static final int OCTOBER = 9;
+
   /**
    * Constant representing November.
    */
   public static final int NOVEMBER = 10;
+
   /**
    * Constant representing December.
    */
   public static final int DECEMBER = 11;
+
   /**
    * Constant representing Undecimber. This is an artificial name useful
    * for lunar calendars.
@@ -280,6 +317,7 @@ public abstract class Calendar implements Serializable, Cloneable
    * Useful constant for 12-hour clock.
    */
   public static final int AM = 0;
+
   /**
    * Useful constant for 12-hour clock.
    */
@@ -291,21 +329,25 @@ public abstract class Calendar implements Serializable, Cloneable
    * @serial
    */
   protected int[] fields = new int[FIELD_COUNT];
+
   /**
    * The flags which tell if the fields above have a value.
    * @serial
    */
   protected boolean[] isSet = new boolean[FIELD_COUNT];
+
   /**
    * The time in milliseconds since the epoch.
    * @serial
    */
   protected long time;
+
   /**
    * Tells if the above field has a valid value.
    * @serial
    */
   protected boolean isTimeSet;
+
   /**
    * Tells if the fields have a valid value.  This superseeds the isSet
    * array.
@@ -331,7 +373,7 @@ public abstract class Calendar implements Serializable, Cloneable
 
   /**
    * Sets what the first day of week is.  This is used for
-   * WEEK_OF_MONTH and WEEK_OF_YEAR fields. 
+   * WEEK_OF_MONTH and WEEK_OF_YEAR fields.
    * @serial
    */
   private int firstDayOfWeek;
@@ -346,13 +388,20 @@ public abstract class Calendar implements Serializable, Cloneable
   private int minimalDaysInFirstWeek;
 
   /**
-   * The version of the serialized data on the stream. 
+   * Is set to true if DST_OFFSET is explicitly set. In that case
+   * it's value overrides the value computed from the current
+   * time and the timezone.
+   */
+  private boolean explicitDSTOffset = false;
+
+  /**
+   * The version of the serialized data on the stream.
    * <dl><dt>0 or not present</dt>
    * <dd> JDK 1.1.5 or later.</dd>
-   * <dl><dt>1</dt>
+   * <dt>1</dt>
    * <dd>JDK 1.1.6 or later.  This always writes a correct `time' value
    * on the stream, as well as the other fields, to be compatible with
-   * earlier versions</dd>
+   * earlier versions</dd></dl>
    * @since JDK1.1.6
    * @serial
    */
@@ -370,13 +419,14 @@ public abstract class Calendar implements Serializable, Cloneable
   private static final String bundleName = "gnu.java.locale.Calendar";
 
   /**
-   * get resource bundle: 
+   * get resource bundle:
    * The resources should be loaded via this method only. Iff an application
-   * uses this method, the resourcebundle is required. 
+   * uses this method, the resourcebundle is required.
    */
-  private static ResourceBundle getBundle(Locale locale) 
+  private static ResourceBundle getBundle(Locale locale)
   {
-    return ResourceBundle.getBundle(bundleName, locale);
+    return ResourceBundle.getBundle(bundleName, locale,
+                                    ClassLoader.getSystemClassLoader());
   }
 
   /**
@@ -402,8 +452,9 @@ public abstract class Calendar implements Serializable, Cloneable
     ResourceBundle rb = getBundle(locale);
 
     firstDayOfWeek = ((Integer) rb.getObject("firstDayOfWeek")).intValue();
-    minimalDaysInFirstWeek =
-      ((Integer) rb.getObject("minimalDaysInFirstWeek")).intValue();
+    minimalDaysInFirstWeek = ((Integer) rb.getObject("minimalDaysInFirstWeek"))
+                             .intValue();
+    clear();
   }
 
   /**
@@ -435,6 +486,18 @@ public abstract class Calendar implements Serializable, Cloneable
     return getInstance(TimeZone.getDefault(), locale);
   }
 
+  /**
+   * Cache of locale->calendar-class mappings. This avoids having to do a ResourceBundle
+   * lookup for every getInstance call.
+   */
+  private static HashMap cache = new HashMap();
+
+  /** Preset argument types for calendar-class constructor lookup.  */
+  private static Class[] ctorArgTypes = new Class[]
+                                        {
+                                          TimeZone.class, Locale.class
+                                        };
+
   /**
    * Creates a calendar representing the actual time, using the given
    * time zone and locale.
@@ -443,29 +506,58 @@ public abstract class Calendar implements Serializable, Cloneable
    */
   public static synchronized Calendar getInstance(TimeZone zone, Locale locale)
   {
-    String calendarClassName = null;
-    ResourceBundle rb = getBundle(locale);
-    calendarClassName = rb.getString("calendarClass");
-    if (calendarClassName != null)
+    Class calendarClass = (Class) cache.get(locale);
+    Throwable exception = null;
+
+    try
       {
-	try
+	if (calendarClass == null)
 	  {
-	    Class calendarClass = Class.forName(calendarClassName);
-	    if (Calendar.class.isAssignableFrom(calendarClass))
+	    ResourceBundle rb = getBundle(locale);
+	    String calendarClassName = rb.getString("calendarClass");
+
+	    if (calendarClassName != null)
 	      {
-		return (Calendar) calendarClass.getConstructor(
-		  new Class[] { TimeZone.class, Locale.class}
-		).newInstance(new Object[] {zone, locale} );
+		calendarClass = Class.forName(calendarClassName);
+		if (Calendar.class.isAssignableFrom(calendarClass))
+		  cache.put(locale, calendarClass);
 	      }
 	  }
-	catch (ClassNotFoundException ex) {}
-	catch (IllegalAccessException ex) {}
-	catch (NoSuchMethodException ex) {}
-	catch (InstantiationException ex) {}
-	catch (InvocationTargetException ex) {}
-	// XXX should we ignore these errors or throw an exception ?
+
+	// GregorianCalendar is by far the most common case. Optimize by 
+	// avoiding reflection.
+	if (calendarClass == GregorianCalendar.class)
+	  return new GregorianCalendar(zone, locale);
+
+	if (Calendar.class.isAssignableFrom(calendarClass))
+	  {
+	    Constructor ctor = calendarClass.getConstructor(ctorArgTypes);
+	    return (Calendar) ctor.newInstance(new Object[] { zone, locale });
+	  }
       }
-    return new GregorianCalendar(zone, locale);
+    catch (ClassNotFoundException ex)
+      {
+	exception = ex;
+      }
+    catch (IllegalAccessException ex)
+      {
+	exception = ex;
+      }
+    catch (NoSuchMethodException ex)
+      {
+	exception = ex;
+      }
+    catch (InstantiationException ex)
+      {
+	exception = ex;
+      }
+    catch (InvocationTargetException ex)
+      {
+	exception = ex;
+      }
+
+    throw new RuntimeException("Error instantiating calendar for locale "
+                               + locale, exception);
   }
 
   /**
@@ -489,7 +581,7 @@ public abstract class Calendar implements Serializable, Cloneable
    * Converts the milliseconds since the epoch UTC
    * (<code>time</code>) to time fields
    * (<code>fields</code>). Override this method if you write your
-   * own Calendar.  
+   * own Calendar.
    */
   protected abstract void computeFields();
 
@@ -500,7 +592,7 @@ public abstract class Calendar implements Serializable, Cloneable
    */
   public final Date getTime()
   {
-    if (!isTimeSet)
+    if (! isTimeSet)
       computeTime();
     return new Date(time);
   }
@@ -521,7 +613,7 @@ public abstract class Calendar implements Serializable, Cloneable
    */
   public long getTimeInMillis()
   {
-    if (!isTimeSet)
+    if (! isTimeSet)
       computeTime();
     return time;
   }
@@ -534,6 +626,7 @@ public abstract class Calendar implements Serializable, Cloneable
    */
   public void setTimeInMillis(long time)
   {
+    clear();
     this.time = time;
     isTimeSet = true;
     computeFields();
@@ -544,24 +637,29 @@ public abstract class Calendar implements Serializable, Cloneable
    * if they are invalid.
    * @param field the time field. One of the time field constants.
    * @return the value of the specified field
-   *
+   * @throws ArrayIndexOutOfBoundsException if the field is outside
+   *         the valid range.  The value of field must be >= 0 and
+   *         <= <code>FIELD_COUNT</code>.
    * @specnote Not final since JDK 1.4
    */
   public int get(int field)
   {
     // If the requested field is invalid, force all fields to be recomputed.
-    if (!isSet[field])
+    if (! isSet[field])
       areFieldsSet = false;
     complete();
     return fields[field];
   }
 
   /**
-   * Gets the value of the specified field. This method doesn't 
+   * Gets the value of the specified field. This method doesn't
    * recompute the fields, if they are invalid.
    * @param field the time field. One of the time field constants.
    * @return the value of the specified field, undefined if
    * <code>areFieldsSet</code> or <code>isSet[field]</code> is false.
+   * @throws ArrayIndexOutOfBoundsException if the field is outside
+   *         the valid range.  The value of field must be >= 0 and
+   *         <= <code>FIELD_COUNT</code>.
    */
   protected final int internalGet(int field)
   {
@@ -573,26 +671,85 @@ public abstract class Calendar implements Serializable, Cloneable
    * the time in milliseconds.
    * @param field the time field. One of the time field constants
    * @param value the value to be set.
-   *
+   * @throws ArrayIndexOutOfBoundsException if field is outside
+   *         the valid range.  The value of field must be >= 0 and
+   *         <= <code>FIELD_COUNT</code>.
    * @specnote Not final since JDK 1.4
    */
   public void set(int field, int value)
   {
+    if (isTimeSet)
+      for (int i = 0; i < FIELD_COUNT; i++)
+	isSet[i] = false;
     isTimeSet = false;
     fields[field] = value;
     isSet[field] = true;
+
+    // The five valid date patterns, in order of priority
+    // 1  YEAR + MONTH + DAY_OF_MONTH
+    // 2  YEAR + MONTH + WEEK_OF_MONTH + DAY_OF_WEEK
+    // 3  YEAR + MONTH + DAY_OF_WEEK_IN_MONTH + DAY_OF_WEEK
+    // 4  YEAR + DAY_OF_YEAR
+    // 5  YEAR + DAY_OF_WEEK + WEEK_OF_YEAR
     switch (field)
       {
-      case YEAR:
-      case MONTH:
-      case DATE:
-	isSet[WEEK_OF_YEAR] = false;
+      case MONTH: // pattern 1,2 or 3
 	isSet[DAY_OF_YEAR] = false;
-	isSet[WEEK_OF_MONTH] = false;
+	isSet[WEEK_OF_YEAR] = false;
+	break;
+      case DAY_OF_MONTH: // pattern 1
+	isSet[YEAR] = true;
+	isSet[MONTH] = true;
+	isSet[WEEK_OF_MONTH] = true;
 	isSet[DAY_OF_WEEK] = false;
 	isSet[DAY_OF_WEEK_IN_MONTH] = false;
+	isSet[DAY_OF_YEAR] = false;
+	isSet[WEEK_OF_YEAR] = false;
+	break;
+      case WEEK_OF_MONTH: // pattern 2
+	if (! isSet[DAY_OF_WEEK])
+	  fields[DAY_OF_WEEK] = getFirstDayOfWeek();
+	isSet[YEAR] = true;
+	isSet[MONTH] = true;
+	isSet[DAY_OF_WEEK] = true;
+	isSet[DAY_OF_MONTH] = false;
+	isSet[DAY_OF_WEEK_IN_MONTH] = false;
+	isSet[DAY_OF_YEAR] = false;
+	isSet[WEEK_OF_YEAR] = false;
+	break;
+      case DAY_OF_WEEK_IN_MONTH: // pattern 3
+	if (! isSet[DAY_OF_WEEK])
+	  fields[DAY_OF_WEEK] = getFirstDayOfWeek();
+	isSet[YEAR] = true;
+	isSet[MONTH] = true;
+	isSet[DAY_OF_WEEK] = true;
+	isSet[DAY_OF_YEAR] = false;
+	isSet[DAY_OF_MONTH] = false;
+	isSet[WEEK_OF_MONTH] = false;
+	isSet[WEEK_OF_YEAR] = false;
+	break;
+      case DAY_OF_YEAR: // pattern 4
+	isSet[YEAR] = true;
+	isSet[MONTH] = false;
+	isSet[WEEK_OF_MONTH] = false;
+	isSet[DAY_OF_MONTH] = false;
+	isSet[DAY_OF_WEEK] = false;
+	isSet[WEEK_OF_YEAR] = false;
+	isSet[DAY_OF_WEEK_IN_MONTH] = false;
+	break;
+      case WEEK_OF_YEAR: // pattern 5
+	if (! isSet[DAY_OF_WEEK])
+	  fields[DAY_OF_WEEK] = getFirstDayOfWeek();
+	isSet[YEAR] = true;
+	isSet[DAY_OF_WEEK] = true;
+	isSet[MONTH] = false;
+	isSet[DAY_OF_MONTH] = false;
+	isSet[WEEK_OF_MONTH] = false;
+	isSet[DAY_OF_YEAR] = false;
+	isSet[DAY_OF_WEEK_IN_MONTH] = false;
 	break;
       case AM_PM:
+	isSet[HOUR] = true;
 	isSet[HOUR_OF_DAY] = false;
 	break;
       case HOUR_OF_DAY:
@@ -600,9 +757,16 @@ public abstract class Calendar implements Serializable, Cloneable
 	isSet[HOUR] = false;
 	break;
       case HOUR:
+	isSet[AM_PM] = true;
 	isSet[HOUR_OF_DAY] = false;
 	break;
+      case DST_OFFSET:
+	explicitDSTOffset = true;
       }
+
+    // May have crossed over a DST boundary.
+    if (! explicitDSTOffset && (field != DST_OFFSET && field != ZONE_OFFSET))
+      isSet[DST_OFFSET] = false;
   }
 
   /**
@@ -623,6 +787,10 @@ public abstract class Calendar implements Serializable, Cloneable
     isSet[WEEK_OF_MONTH] = false;
     isSet[DAY_OF_WEEK] = false;
     isSet[DAY_OF_WEEK_IN_MONTH] = false;
+    isSet[ERA] = false;
+
+    if (! explicitDSTOffset)
+      isSet[DST_OFFSET] = false; // May have crossed a DST boundary.
   }
 
   /**
@@ -652,8 +820,8 @@ public abstract class Calendar implements Serializable, Cloneable
    * @param minute the minute.
    * @param second the second.
    */
-  public final void set(int year, int month, int date,
-			int hour, int minute, int second)
+  public final void set(int year, int month, int date, int hour, int minute,
+                        int second)
   {
     set(year, month, date, hour, minute);
     fields[SECOND] = second;
@@ -667,28 +835,43 @@ public abstract class Calendar implements Serializable, Cloneable
   {
     isTimeSet = false;
     areFieldsSet = false;
+    int zoneOffs = zone.getRawOffset();
+    int[] tempFields = 
+                       {
+                         1, 1970, JANUARY, 1, 1, 1, 1, THURSDAY, 1, AM, 0, 0, 0,
+                         0, 0, zoneOffs, 0
+                       };
+    fields = tempFields;
     for (int i = 0; i < FIELD_COUNT; i++)
-      {
-	isSet[i] = false;
-	fields[i] = 0;
-      }
+      isSet[i] = false;
   }
 
   /**
    * Clears the values of the specified time field.
    * @param field the time field. One of the time field constants.
+   * @throws ArrayIndexOutOfBoundsException if field is outside
+   *         the valid range.  The value of field must be >= 0 and
+   *         <= <code>FIELD_COUNT</code>.
    */
   public final void clear(int field)
   {
+    int[] tempFields = 
+                       {
+                         1, 1970, JANUARY, 1, 1, 1, 1, THURSDAY, 1, AM, 0, 0, 0,
+                         0, 0, zone.getRawOffset(), 0
+                       };
     isTimeSet = false;
     areFieldsSet = false;
     isSet[field] = false;
-    fields[field] = 0;
+    fields[field] = tempFields[field];
   }
 
   /**
    * Determines if the specified field has a valid value.
    * @return true if the specified field has a value.
+   * @throws ArrayIndexOutOfBoundsException if the field is outside
+   *         the valid range.  The value of field must be >= 0 and
+   *         <= <code>FIELD_COUNT</code>.
    */
   public final boolean isSet(int field)
   {
@@ -697,18 +880,18 @@ public abstract class Calendar implements Serializable, Cloneable
 
   /**
    * Fills any unset fields in the time field list
-   * @return true if the specified field has a value.  
+   * @return true if the specified field has a value.
    */
   protected void complete()
   {
-    if (!isTimeSet)
+    if (! isTimeSet)
       computeTime();
-    if (!areFieldsSet)
+    if (! areFieldsSet)
       computeFields();
   }
 
   /**
-   * Compares the given calendar with this.  
+   * Compares the given calendar with this.
    * @param o the object to that we should compare.
    * @return true, if the given object is a calendar, that represents
    * the same time (but doesn't necessary have the same fields).
@@ -716,12 +899,12 @@ public abstract class Calendar implements Serializable, Cloneable
   public boolean equals(Object o)
   {
     return (o instanceof Calendar)
-      && getTimeInMillis() == ((Calendar) o).getTimeInMillis();
+           && getTimeInMillis() == ((Calendar) o).getTimeInMillis();
   }
 
   /**
    * Returns a hash code for this calendar.
-   * @return a hash code, which fullfits the general contract of 
+   * @return a hash code, which fullfits the general contract of
    * <code>hashCode()</code>
    */
   public int hashCode()
@@ -731,7 +914,7 @@ public abstract class Calendar implements Serializable, Cloneable
   }
 
   /**
-   * Compares the given calendar with this.  
+   * Compares the given calendar with this.
    * @param o the object to that we should compare.
    * @return true, if the given object is a calendar, and this calendar
    * represents a smaller time than the calendar o.
@@ -744,7 +927,7 @@ public abstract class Calendar implements Serializable, Cloneable
   }
 
   /**
-   * Compares the given calendar with this.  
+   * Compares the given calendar with this.
    * @param o the object to that we should compare.
    * @return true, if the given object is a calendar, and this calendar
    * represents a bigger time than the calendar o.
@@ -762,20 +945,26 @@ public abstract class Calendar implements Serializable, Cloneable
    * it does what you expect: Jan, 25 + 10 Days is Feb, 4.
    * @param field the time field. One of the time field constants.
    * @param amount the amount of time.
+   * @throws ArrayIndexOutOfBoundsException if the field is outside
+   *         the valid range.  The value of field must be >= 0 and
+   *         <= <code>FIELD_COUNT</code>.
    */
   public abstract void add(int field, int amount);
 
   /**
    * Rolls the specified time field up or down.  This means add one
    * to the specified field, but don't change the other fields.  If
-   * the maximum for this field is reached, start over with the 
+   * the maximum for this field is reached, start over with the
    * minimum value.  <br>
    *
    * <strong>Note:</strong> There may be situation, where the other
-   * fields must be changed, e.g rolling the month on May, 31. 
+   * fields must be changed, e.g rolling the month on May, 31.
    * The date June, 31 is automatically converted to July, 1.
    * @param field the time field. One of the time field constants.
    * @param up the direction, true for up, false for down.
+   * @throws ArrayIndexOutOfBoundsException if the field is outside
+   *         the valid range.  The value of field must be >= 0 and
+   *         <= <code>FIELD_COUNT</code>.
    */
   public abstract void roll(int field, boolean up);
 
@@ -788,7 +977,10 @@ public abstract class Calendar implements Serializable, Cloneable
    *
    * @param field the time field. One of the time field constants.
    * @param amount the amount to roll by, positive for rolling up,
-   * negative for rolling down.  
+   * negative for rolling down.
+   * @throws ArrayIndexOutOfBoundsException if the field is outside
+   *         the valid range.  The value of field must be >= 0 and
+   *         <= <code>FIELD_COUNT</code>.
    * @since JDK1.2
    */
   public void roll(int field, int amount)
@@ -805,7 +997,6 @@ public abstract class Calendar implements Serializable, Cloneable
       }
   }
 
-
   /**
    * Sets the time zone to the specified value.
    * @param zone the new time zone
@@ -849,7 +1040,7 @@ public abstract class Calendar implements Serializable, Cloneable
 
   /**
    * Sets what the first day of week is.  This is used for
-   * WEEK_OF_MONTH and WEEK_OF_YEAR fields. 
+   * WEEK_OF_MONTH and WEEK_OF_YEAR fields.
    * @param value the first day of week.  One of SUNDAY to SATURDAY.
    */
   public void setFirstDayOfWeek(int value)
@@ -859,7 +1050,7 @@ public abstract class Calendar implements Serializable, Cloneable
 
   /**
    * Gets what the first day of week is.  This is used for
-   * WEEK_OF_MONTH and WEEK_OF_YEAR fields. 
+   * WEEK_OF_MONTH and WEEK_OF_YEAR fields.
    * @return the first day of week.  One of SUNDAY to SATURDAY.
    */
   public int getFirstDayOfWeek()
@@ -903,7 +1094,6 @@ public abstract class Calendar implements Serializable, Cloneable
    */
   public abstract int getMaximum(int field);
 
-
   /**
    * Gets the greatest minimum value that is allowed for the specified field.
    * @param field the time field. One of the time field constants.
@@ -915,7 +1105,7 @@ public abstract class Calendar implements Serializable, Cloneable
    * Gets the smallest maximum value that is allowed for the
    * specified field.  For example this is 28 for DAY_OF_MONTH.
    * @param field the time field. One of the time field constants.
-   * @return the least maximum value.  
+   * @return the least maximum value.
    */
   public abstract int getLeastMaximum(int field);
 
@@ -924,20 +1114,50 @@ public abstract class Calendar implements Serializable, Cloneable
    * This value is dependent on the values of the other fields.
    * @param field the time field. One of the time field constants.
    * @return the actual minimum value.
+   * @throws ArrayIndexOutOfBoundsException if the field is outside
+   *         the valid range.  The value of field must be >= 0 and
+   *         <= <code>FIELD_COUNT</code>.
    * @since jdk1.2
    */
-  // FIXME: XXX: Not abstract in JDK 1.2.
-  public abstract int getActualMinimum(int field);
+  public int getActualMinimum(int field)
+  {
+    Calendar tmp = (Calendar) clone(); // To avoid restoring state
+    int min = tmp.getGreatestMinimum(field);
+    int end = tmp.getMinimum(field);
+    tmp.set(field, min);
+    for (; min > end; min--)
+      {
+	tmp.add(field, -1); // Try to get smaller
+	if (tmp.get(field) != min - 1)
+	  break; // Done if not successful
+      }
+    return min;
+  }
 
   /**
    * Gets the actual maximum value that is allowed for the specified field.
    * This value is dependent on the values of the other fields.
    * @param field the time field. One of the time field constants.
-   * @return the actual maximum value.  
+   * @return the actual maximum value.
+   * @throws ArrayIndexOutOfBoundsException if the field is outside
+   *         the valid range.  The value of field must be >= 0 and
+   *         <= <code>FIELD_COUNT</code>.
    * @since jdk1.2
    */
-  // FIXME: XXX: Not abstract in JDK 1.2.
-  public abstract int getActualMaximum(int field);
+  public int getActualMaximum(int field)
+  {
+    Calendar tmp = (Calendar) clone(); // To avoid restoring state
+    int max = tmp.getLeastMaximum(field);
+    int end = tmp.getMaximum(field);
+    tmp.set(field, max);
+    for (; max < end; max++)
+      {
+	tmp.add(field, 1);
+	if (tmp.get(field) != max + 1)
+	  break;
+      }
+    return max;
+  }
 
   /**
    * Return a clone of this object.
@@ -948,7 +1168,7 @@ public abstract class Calendar implements Serializable, Cloneable
       {
 	Calendar cal = (Calendar) super.clone();
 	cal.fields = (int[]) fields.clone();
-	cal.isSet = (boolean[])isSet.clone();
+	cal.isSet = (boolean[]) isSet.clone();
 	return cal;
       }
     catch (CloneNotSupportedException ex)
@@ -957,16 +1177,19 @@ public abstract class Calendar implements Serializable, Cloneable
       }
   }
 
-  private final static String[] fieldNames = {
-    ",ERA=", ",YEAR=", ",MONTH=",
-    ",WEEK_OF_YEAR=", ",WEEK_OF_MONTH=",
-    ",DAY_OF_MONTH=", ",DAY_OF_YEAR=", ",DAY_OF_WEEK=",
-    ",DAY_OF_WEEK_IN_MONTH=",
-    ",AM_PM=", ",HOUR=", ",HOUR_OF_DAY=",
-    ",MINUTE=", ",SECOND=", ",MILLISECOND=",
-    ",ZONE_OFFSET=", ",DST_OFFSET="
-  };
-
+  private static final String[] fieldNames = 
+                                             {
+                                               ",ERA=", ",YEAR=", ",MONTH=",
+                                               ",WEEK_OF_YEAR=",
+                                               ",WEEK_OF_MONTH=",
+                                               ",DAY_OF_MONTH=",
+                                               ",DAY_OF_YEAR=", ",DAY_OF_WEEK=",
+                                               ",DAY_OF_WEEK_IN_MONTH=",
+                                               ",AM_PM=", ",HOUR=",
+                                               ",HOUR_OF_DAY=", ",MINUTE=",
+                                               ",SECOND=", ",MILLISECOND=",
+                                               ",ZONE_OFFSET=", ",DST_OFFSET="
+                                             };
 
   /**
    * Returns a string representation of this object.  It is mainly
@@ -1009,7 +1232,7 @@ public abstract class Calendar implements Serializable, Cloneable
    * says, that it could be omitted.  */
   private void writeObject(ObjectOutputStream stream) throws IOException
   {
-    if (!isTimeSet)
+    if (! isTimeSet)
       computeTime();
     stream.defaultWriteObject();
   }
@@ -1021,7 +1244,7 @@ public abstract class Calendar implements Serializable, Cloneable
     throws IOException, ClassNotFoundException
   {
     stream.defaultReadObject();
-    if (!isTimeSet)
+    if (! isTimeSet)
       computeTime();
 
     if (serialVersionOnStream > 1)
@@ -1030,7 +1253,6 @@ public abstract class Calendar implements Serializable, Cloneable
 	// Sun wants to remove all fields from the stream someday
 	// and will then increase the serialVersion number again.
 	// We prepare to be compatible.
-
 	fields = new int[FIELD_COUNT];
 	isSet = new boolean[FIELD_COUNT];
 	areFieldsSet = false;
