@@ -1,5 +1,5 @@
-/* java.util.zip.ZipOutputStream
-   Copyright (C) 2001 Free Software Foundation, Inc.
+/* ZipOutputStream.java --
+   Copyright (C) 2001, 2004  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -35,12 +35,13 @@ this exception to your version of the library, but you are not
 obligated to do so.  If you do not wish to do so, delete this
 exception statement from your version. */
 
+
 package java.util.zip;
-import java.io.OutputStream;
+
 import java.io.IOException;
-import java.io.UnsupportedEncodingException;
-import java.util.Vector;
+import java.io.OutputStream;
 import java.util.Enumeration;
+import java.util.Vector;
 
 /**
  * This is a FilterOutputStream that writes the files into a zip
@@ -70,17 +71,18 @@ public class ZipOutputStream extends DeflaterOutputStream implements ZipConstant
   /**
    * Our Zip version is hard coded to 1.0 resp. 2.0
    */
-  private final static int ZIP_STORED_VERSION   = 10;
-  private final static int ZIP_DEFLATED_VERSION = 20;
+  private static final int ZIP_STORED_VERSION = 10;
+  private static final int ZIP_DEFLATED_VERSION = 20;
 
   /**
    * Compression method.  This method doesn't compress at all.
    */
-  public final static int STORED      =  0;
+  public static final int STORED = 0;
+  
   /**
    * Compression method.  This method uses the Deflater.
    */
-  public final static int DEFLATED    =  8;
+  public static final int DEFLATED = 8;
 
   /**
    * Creates a new Zip output stream, writing a zip archive.
@@ -135,7 +137,7 @@ public class ZipOutputStream extends DeflaterOutputStream implements ZipConstant
   /**
    * Write an unsigned short in little endian byte order.
    */
-  private final void writeLeShort(int value) throws IOException 
+  private void writeLeShort(int value) throws IOException 
   {
     out.write(value & 0xff);
     out.write((value >> 8) & 0xff);
@@ -144,7 +146,7 @@ public class ZipOutputStream extends DeflaterOutputStream implements ZipConstant
   /**
    * Write an int in little endian byte order.
    */
-  private final void writeLeInt(int value) throws IOException 
+  private void writeLeInt(int value) throws IOException 
   {
     writeLeShort(value);
     writeLeShort(value >> 16);
@@ -337,10 +339,10 @@ public class ZipOutputStream extends DeflaterOutputStream implements ZipConstant
     int numEntries = 0;
     int sizeEntries = 0;
     
-    Enumeration enum = entries.elements();
-    while (enum.hasMoreElements())
+    Enumeration e = entries.elements();
+    while (e.hasMoreElements())
       {
-	ZipEntry entry = (ZipEntry) enum.nextElement();
+	ZipEntry entry = (ZipEntry) e.nextElement();
 	
 	int method = entry.getMethod();
 	writeLeInt(CENSIG);
